@@ -3,6 +3,7 @@ from __future__ import annotations
 import logging
 import os
 import subprocess
+import sys
 from contextlib import nullcontext
 from datetime import datetime
 from typing import TYPE_CHECKING
@@ -31,7 +32,7 @@ from semantic_release.version import Version, next_version, tags_and_versions
 log = logging.getLogger(__name__)
 
 if TYPE_CHECKING:  # pragma: no cover
-    from typing import ContextManager, Iterable
+    from typing import ContextManager, Iterable, Mapping
 
     from git import Repo
     from git.refs.tag import Tag
@@ -135,7 +136,9 @@ def apply_version_to_source_files(
     return paths
 
 
-def shell(cmd: str, *, check: bool = True) -> subprocess.CompletedProcess:
+def shell(
+    cmd: str, *, env: Mapping[str, str] | None = None, check: bool = True
+) -> subprocess.CompletedProcess:
     shell: str | None
     try:
         shell, _ = shellingham.detect_shell()
@@ -149,10 +152,45 @@ def shell(cmd: str, *, check: bool = True) -> subprocess.CompletedProcess:
 
     return subprocess.run(
         [shell, "-c" if shell != "cmd" else "/c", cmd],  # noqa: S603
+        env=(env or {}),
         check=check,
     )
 
 
+def is_windows() -> bool:
+    return sys.platform == "win32"
+
+
+def get_windows_env() -> Mapping[str, str | None]:
+    return {
+        environment_variable: os.getenv(environment_variable, None)
+        for environment_variable in (
+            "ALLUSERSAPPDATA",
+            "ALLUSERSPROFILE",
+            "APPDATA",
+            "COMMONPROGRAMFILES",
+            "COMMONPROGRAMFILES(x86)",
+            "DEFAULTUSERPROFILE",
+            "HOMEPATH",
+            "PATHEXT",
+            "PROFILESFOLDER",
+            "PROGRAMFILES",
+            "PROGRAMFILES(x86)",
+            "SYSTEM",
+            "SYSTEM16",
+            "SYSTEM32",
+            "SYSTEMDRIVE",
+            "SYSTEMPROFILE",
+            "SYSTEMROOT",
+            "TEMP",
+            "TMP",
+            "USERPROFILE",
+            "USERSID",
+            "WINDIR",
+        )
+    }
+
+
 @click.command(
     short_help="Detect and apply a new version",
     context_settings={
@@ -327,6 +365,7 @@ def version(  # noqa: C901
     commit_author = runtime.commit_author
     commit_message = runtime.commit_message
     major_on_zero = runtime.major_on_zero
+    no_verify = runtime.no_git_verify
     build_command = runtime.build_command
     opts = runtime.global_cli_options
     gha_output = VersionGitHubActionsOutput()
@@ -507,7 +546,41 @@ def version(  # noqa: C901
                 "[bold green]:hammer_and_wrench: Running build command: "
                 + build_command
             )
-            shell(build_command, check=True)
+            shell(
+                build_command,
+                check=True,
+                env=dict(
+                    filter(
+                        lambda k_v: k_v[1] is not None,  # type: ignore
+                        {
+                            # Common values
+                            "PATH": os.getenv("PATH", ""),
+                            "HOME": os.getenv("HOME", None),
+                            "VIRTUAL_ENV": os.getenv("VIRTUAL_ENV", None),
+                            # Windows environment variables
+                            **(get_windows_env() if is_windows() else {}),
+                            # affects build decisions
+                            "CI": os.getenv("CI", None),
+                            # Identifies which CI environment
+                            "GITHUB_ACTIONS": os.getenv("GITHUB_ACTIONS", None),
+                            "GITLAB_CI": os.getenv("GITLAB_CI", None),
+                            "GITEA_ACTIONS": os.getenv("GITEA_ACTIONS", None),
+                            "BITBUCKET_CI": (
+                                str(True).lower()
+                                if os.getenv("BITBUCKET_REPO_FULL_NAME", None)
+                                else None
+                            ),
+                            "PSR_DOCKER_GITHUB_ACTION": os.getenv(
+                                "PSR_DOCKER_GITHUB_ACTION", None
+                            ),
+                            # User defined overrides of environment (from config)
+                            **runtime.build_command_env,
+                            # PSR injected environment variables
+                            "NEW_VERSION": str(new_version),
+                        }.items(),
+                    )
+                ),
+            )
         except subprocess.CalledProcessError as exc:
             ctx.fail(str(exc))
 
@@ -578,6 +651,7 @@ def version(  # noqa: C901
         )
 
         command += f"git commit -m '{indented_commit_message}'"
+        command += "--no-verify" if no_verify else ""
 
         noop_report(
             indented(
@@ -593,6 +667,7 @@ def version(  # noqa: C901
             repo.git.commit(
                 m=commit_message.format(version=new_version),
                 date=int(commit_date.timestamp()),
+                no_verify=no_verify,
             )
 
     # Run the tagging after potentially creating a new HEAD commit.
